﻿using UnityEngine;
using System.Collections.Generic;

public class Prey : Echoable {

    BarProgress m_fearBar;
    NavMeshAgent m_agent;

    float m_fear = 0, m_maxFear = 10, m_targetFear;
    float m_fearGravity = 0;

    List<Echo> m_knownEchos = new List<Echo>();

    public float m_fearIncreaseMultiplier = 10;
    public float m_fearBrakingMultiplier = 2.5f;
    public float m_revealInterval = 1000;

    bool m_tempReveal = false;
    bool m_tRFade = false;
    float m_tRAlpha = 0;
    int m_tRCycles = 1, m_tRMaxCycles = 3;
    Timer m_tRTimer;

    Timer m_runAwayTimer;

    Vector3 m_lastPos, m_targetPos;
    int m_failedMoves = 0;
    bool m_resetMovement = false;

    // Use this for initialization
    protected override void Start () {

        
        m_fearBar = GameObject.Find("bpFear").GetComponent<BarProgress>();
        m_agent = GetComponent<NavMeshAgent>();

        m_runAwayTimer = gameObject.AddComponent<Timer>();
        m_runAwayTimer.PushCallback(RunAway);
        m_tRTimer = gameObject.AddComponent<Timer>();
        m_tRTimer.PushCallback(TempReveal);

        base.Start();

        Show();
        RunAway();
    }
	
	// Update is called once per frame
	protected override void Update () {
        base.Update();

        UpdateFear();
        UpdateReveals();
        UpdatePlayerDetection();
        
        if(m_resetMovement)
        {
            if (Vector3.Distance(transform.position, m_targetPos) <= 1)
                m_resetMovement = false;
        }

        if (Input.GetKeyDown(KeyCode.LeftShift))
            m_agent.destination = Vector3.zero;
    }

    void UpdateReveals()
    {
        if (m_tempReveal)
        {
            if (!m_tRFade)
            {
                m_tRAlpha = 1;
                m_tRFade = true;
            }
            else
            {
                if (m_tRAlpha > 0)
                    m_tRAlpha -= 3 * Time.deltaTime;
                else
                {
                    if (m_tRCycles < m_tRMaxCycles)
                    {
                        m_tRFade = false;
                        m_tRCycles++;
                    }
                    else
                    {
                        m_tempReveal = false;
                        m_tRCycles = 0;
                        Hide();

                        if (m_fear < m_maxFear)
                            m_tRTimer.Start(m_revealInterval * (1.5f - (m_fear / m_maxFear)));
                    }
                }
            }

            foreach (MeshRenderer mr in m_renderers)
            {
                Color col = mr.material.color;

                col.a = m_tRAlpha;

                mr.material.color = col;
            }
        }
    }
    void UpdateFear()
    {
        m_fearBar.barFill = (m_fear / m_maxFear);

        m_fear = Mathf.Lerp(m_fear, m_targetFear, Time.deltaTime * .5f);

        m_targetFear += m_fearGravity;

        m_fearGravity = Mathf.Lerp(m_fearGravity, 0, Time.deltaTime * (m_fearBrakingMultiplier * (1 - GetFearMultiplier() * .5f)));

        if (m_fear >= m_maxFear)
            Kill();
    }

    void UpdatePlayerDetection()
    {
        if (!m_player.isWalking && Vector3.Distance(transform.position, m_player.transform.position) < 5)
            RunAway();
    }

    void RunAway()
    {
        //..test to see if the prey has moved or not
        if (Vector3.Distance(transform.position, m_lastPos) < 1)
        {
            if (m_failedMoves < 3)
                m_failedMoves++;
            else
            {
                m_targetPos = transform.position * -1;
                m_agent.destination = m_targetPos;
                m_resetMovement = true;
            }
        }

        if (!m_resetMovement)
        {
            Vector3 runDir = (m_player.transform.position - transform.position).normalized;

            runDir.y = 0;

            float offset = Random.value * 90;

            runDir.x = runDir.x - Mathf.Cos(offset * Mathf.Deg2Rad) * 1;
            runDir.z = runDir.z - Mathf.Sin(offset * Mathf.Deg2Rad) * 1;

            m_agent.destination = -runDir * 10;

            Debug.DrawRay(transform.position, -runDir, Color.red);

            m_lastPos = transform.position;
        }

        m_runAwayTimer.Start(1000);
    }

    //..Amplify a value based on how scared the prey is. The more scared the prey is, the higher this value will be.
    float GetFearMultiplier()
    {
        return 1 + (m_fearGravity / (m_maxFear * .5f));
    }

    //..Amplify a value based on how far away the prey is from the monster. The closer the player is, the greater the effect
    float GetDistanceMultiplier()
    {
        float falloff = m_fearIncreaseMultiplier;
        float dist = Vector3.Distance(transform.position, m_player.transform.position);
        float difference = dist - falloff;

        return (falloff / dist);
    }

    protected override void Show()
    {
        base.Show();
    }

    public void Scare(Echo a_echo)
    {
        if(!m_knownEchos.Contains(a_echo)) //..Make sure that the same echo won't scare the prey twice.
        {
            m_fearGravity += .05f * GetDistanceMultiplier() * GetFearMultiplier();
            m_knownEchos.Add(a_echo);
            m_tRTimer.Start(3000 * (1.5f - (m_fear / m_maxFear)));

            foreach (MeshRenderer mr in m_renderers)
            {
                Color col = mr.material.color;

                col.a = m_tRAlpha;

                mr.material.color = col;
            }
        }
    }

    public void TempReveal()
    {
        m_tempReveal = true;
        m_tRCycles = 0;
        m_tRFade = false;

        foreach(MeshRenderer mr in m_renderers)
            mr.enabled = true;
    }

    public void Kill()
    {
        Destroy(gameObject);
        Debug.Log("The prey has been killed!");
    }
}
